#! /bin/bash
#
# Manual compile of ZMC, DZMC and toolbox programs.
#
# This is for Linux/Mac (POSIX-type) environments.
# I suggest using 'compile_all.bat' for windows.
#
# Dependencies are accounted for by the order of the
# compilations -- some modules depend on others.
# This means that this could be broken by upstream changes
# to the modules by Aidan Heerdegen (aidan@rsc.anu.edu.au)
#
# However, if this does not happen, compilation is simple.
#
# NOTE!!: Some manual customisation of this file will
# probably be needed, but it is very simple:
#
# Namely the -I and -L flags to g95 need to point
# to the directory where the compilation is happening,
# and that it is easiest for a relatively simple project
# like this to use the flat structure, then just copy out
# the executables to a directory in the path.
#
# No guarantees/warranties are given or implied.  Use at
# your own risk.  Caveat emptor.  Buyer beware.
#
# And you get what you pay for...
#
#  goossens@rsc.anu.edu.au, 13 Jan 2012 (A Friday...)
#
echo ---------------------------
COMPILEDIR=`pwd`
rm *.o *.O *.mod *.MOD
echo Compiling in $COMPILEDIR
bindir=ZMCMac
echo ---------------------------
mkdir -v $bindir
echo ---------------------------
#
set -x verbose
g95 -O2   -ffree-line-length-huge -c fundamental_constants.f90
g95 -O2  -ffree-line-length-huge -c varying_string.f90
g95 -O2  -ffree-line-length-huge -c globals.f90
g95 -O2  -ffree-line-length-huge -c precision.f90
g95 -O2  -ffree-line-length-huge -c cartesian_class.f90
g95 -O2  -ffree-line-length-huge -c array_functions.f90

g95 -O2  -ffree-line-length-huge -c sort_functions.f90
g95 -O2  -ffree-line-length-huge -c variable_array.f90
g95 -O2  -ffree-line-length-huge -c binary.f90

g95 -O2  -ffree-line-length-huge -c string_functions.f90
g95 -O2  -ffree-line-length-huge -c statistics.f90

g95 -O2  -ffree-line-length-huge -c binary_io.f90

g95 -O2  -ffree-line-length-huge -c file_functions.f90

g95 -O2  -ffree-line-length-huge -c hash_table.f90
g95 -O2  -ffree-line-length-huge -c polysample.f90
g95 -O2  -ffree-line-length-huge -c vector_class.f90

g95 -O2  -ffree-line-length-huge -c mol2_class.f90
g95 -O2  -ffree-line-length-huge -c cmdline_arguments.f90
g95 -O2  -ffree-line-length-huge -c keyword_class.f90
g95 -O2  -ffree-line-length-huge -c rotmatrix_class.f90

g95 -O2  -ffree-line-length-huge -c crystallography_class.f90
g95 -O2  -ffree-line-length-huge -c image_transforms.f90
g95 -O2  -ffree-line-length-huge -c quaternion_class.f90
g95 -O2  -ffree-line-length-huge -c zmatrix_class.f90

g95 -O2  -ffree-line-length-huge -c pnm_class.f90
g95 -O2  -ffree-line-length-huge -c superimpose.f90
g95 -O2  -fstatic    -o ZMC zmc_Aug24_2012.f90  rannum.f ps_routines.f fundamental_constants.o varying_string.o globals.o precision.o cartesian_class.o array_functions.o sort_functions.o variable_array.o binary.o binary_io.o string_functions.o statistics.o file_functions.o hash_table.o polysample.o vector_class.o mol2_class.o cmdline_arguments.o keyword_class.o rotmatrix_class.o crystallography_class.o image_transforms.o quaternion_class.o zmatrix_class.o superimpose.o pnm_class.o
g95 -O2  -fstatic    -o DZMC  readat_zmc_April20_2011.f90 diffuse_allocatable.f90 rannum.f  ps_routines.f fundamental_constants.o varying_string.o globals.o precision.o cartesian_class.o array_functions.o sort_functions.o variable_array.o binary.o binary_io.o string_functions.o statistics.o file_functions.o hash_table.o polysample.o vector_class.o mol2_class.o cmdline_arguments.o keyword_class.o rotmatrix_class.o crystallography_class.o image_transforms.o quaternion_class.o zmatrix_class.o superimpose.o pnm_class.o
g95 -O2  -fstatic    -o bin2gray   bin2gray.f90   ps_routines.f fundamental_constants.o varying_string.o globals.o precision.o cartesian_class.o array_functions.o sort_functions.o variable_array.o binary.o binary_io.o string_functions.o statistics.o file_functions.o hash_table.o polysample.o vector_class.o mol2_class.o cmdline_arguments.o keyword_class.o rotmatrix_class.o crystallography_class.o image_transforms.o quaternion_class.o zmatrix_class.o superimpose.o pnm_class.o 
g95 -O2  -fstatic    -o zmat_maker  zmat_maker.f90   ps_routines.f fundamental_constants.o varying_string.o globals.o precision.o cartesian_class.o array_functions.o sort_functions.o variable_array.o binary.o binary_io.o string_functions.o statistics.o file_functions.o hash_table.o polysample.o vector_class.o mol2_class.o cmdline_arguments.o keyword_class.o rotmatrix_class.o crystallography_class.o image_transforms.o quaternion_class.o zmatrix_class.o superimpose.o pnm_class.o 
g95 -O2  -fstatic    -o zmat2xyz  zmat2xyz.f90   ps_routines.f fundamental_constants.o varying_string.o globals.o precision.o cartesian_class.o array_functions.o sort_functions.o variable_array.o binary.o binary_io.o string_functions.o statistics.o file_functions.o hash_table.o polysample.o vector_class.o mol2_class.o cmdline_arguments.o keyword_class.o rotmatrix_class.o crystallography_class.o image_transforms.o quaternion_class.o zmatrix_class.o superimpose.o pnm_class.o 
g95 -O2  -fstatic    -o zmatchk  zmatchk.f90   ps_routines.f fundamental_constants.o varying_string.o globals.o precision.o cartesian_class.o array_functions.o sort_functions.o variable_array.o binary.o binary_io.o string_functions.o statistics.o file_functions.o hash_table.o polysample.o vector_class.o mol2_class.o cmdline_arguments.o keyword_class.o rotmatrix_class.o crystallography_class.o image_transforms.o quaternion_class.o zmatrix_class.o superimpose.o pnm_class.o 
g95 -O2  -fstatic    -o zmat_anim  zmat_anim.f90 rannum.f  ps_routines.f fundamental_constants.o varying_string.o globals.o precision.o cartesian_class.o array_functions.o sort_functions.o variable_array.o binary.o binary_io.o string_functions.o statistics.o file_functions.o hash_table.o polysample.o vector_class.o mol2_class.o cmdline_arguments.o keyword_class.o rotmatrix_class.o crystallography_class.o image_transforms.o quaternion_class.o zmatrix_class.o superimpose.o pnm_class.o 
g95 -O2  -fstatic    -o zmat2mol2  zmat2mol2.f90   ps_routines.f fundamental_constants.o varying_string.o globals.o precision.o cartesian_class.o array_functions.o sort_functions.o variable_array.o binary.o binary_io.o string_functions.o statistics.o file_functions.o hash_table.o polysample.o vector_class.o mol2_class.o cmdline_arguments.o keyword_class.o rotmatrix_class.o crystallography_class.o image_transforms.o quaternion_class.o zmatrix_class.o superimpose.o pnm_class.o 
g95 -O2  -fstatic    -o pgmave  pgmave.f90   ps_routines.f fundamental_constants.o varying_string.o globals.o precision.o cartesian_class.o array_functions.o sort_functions.o variable_array.o binary.o binary_io.o string_functions.o statistics.o file_functions.o hash_table.o polysample.o vector_class.o mol2_class.o cmdline_arguments.o keyword_class.o rotmatrix_class.o crystallography_class.o image_transforms.o quaternion_class.o zmatrix_class.o superimpose.o pnm_class.o 
g95 -O2  -fstatic    -o pgmcombine  pgmcombine.f90   ps_routines.f fundamental_constants.o varying_string.o globals.o precision.o cartesian_class.o array_functions.o sort_functions.o variable_array.o binary.o binary_io.o string_functions.o statistics.o file_functions.o hash_table.o polysample.o vector_class.o mol2_class.o cmdline_arguments.o keyword_class.o rotmatrix_class.o crystallography_class.o image_transforms.o quaternion_class.o zmatrix_class.o superimpose.o pnm_class.o 
g95 -O2  -fstatic    -o make_random_occ  make_random_occ.f90 rannum.f  ps_routines.f fundamental_constants.o varying_string.o globals.o precision.o cartesian_class.o array_functions.o sort_functions.o variable_array.o binary.o binary_io.o string_functions.o statistics.o file_functions.o hash_table.o polysample.o vector_class.o mol2_class.o cmdline_arguments.o keyword_class.o rotmatrix_class.o crystallography_class.o image_transforms.o quaternion_class.o zmatrix_class.o superimpose.o pnm_class.o 
g95 -O2  -fstatic    -o mol2xyz  mol2xyz.f90   ps_routines.f fundamental_constants.o varying_string.o globals.o precision.o cartesian_class.o array_functions.o sort_functions.o variable_array.o binary.o binary_io.o string_functions.o statistics.o file_functions.o hash_table.o polysample.o vector_class.o mol2_class.o cmdline_arguments.o keyword_class.o rotmatrix_class.o crystallography_class.o image_transforms.o quaternion_class.o zmatrix_class.o superimpose.o pnm_class.o
g95 -O2  -fstatic    -o catmol2  catmol2.f90   ps_routines.f fundamental_constants.o varying_string.o globals.o precision.o cartesian_class.o array_functions.o sort_functions.o variable_array.o binary.o binary_io.o string_functions.o statistics.o file_functions.o hash_table.o polysample.o vector_class.o mol2_class.o cmdline_arguments.o keyword_class.o rotmatrix_class.o crystallography_class.o image_transforms.o quaternion_class.o zmatrix_class.o superimpose.o pnm_class.o
g95 -O2  -fstatic    -o chkmol2  chkmol2.f90   ps_routines.f fundamental_constants.o varying_string.o globals.o precision.o cartesian_class.o array_functions.o sort_functions.o variable_array.o binary.o binary_io.o string_functions.o statistics.o file_functions.o hash_table.o polysample.o vector_class.o mol2_class.o cmdline_arguments.o keyword_class.o rotmatrix_class.o crystallography_class.o image_transforms.o quaternion_class.o zmatrix_class.o superimpose.o pnm_class.o
g95 -O2  -fstatic    -o pgm2mask  pgm2mask.f90   ps_routines.f fundamental_constants.o varying_string.o globals.o precision.o cartesian_class.o array_functions.o sort_functions.o variable_array.o binary.o binary_io.o string_functions.o statistics.o file_functions.o hash_table.o polysample.o vector_class.o mol2_class.o cmdline_arguments.o keyword_class.o rotmatrix_class.o crystallography_class.o image_transforms.o quaternion_class.o zmatrix_class.o superimpose.o pnm_class.o
g95 -O2  -fstatic    -o pgm2ni  pgm2ni.f90   ps_routines.f fundamental_constants.o varying_string.o globals.o precision.o cartesian_class.o array_functions.o sort_functions.o variable_array.o binary.o binary_io.o string_functions.o statistics.o file_functions.o hash_table.o polysample.o vector_class.o mol2_class.o cmdline_arguments.o keyword_class.o rotmatrix_class.o crystallography_class.o image_transforms.o quaternion_class.o zmatrix_class.o superimpose.o pnm_class.o
g95 -O2  -fstatic    -o ni2pgm  ni2pgm.f90   ps_routines.f fundamental_constants.o varying_string.o globals.o precision.o cartesian_class.o array_functions.o sort_functions.o variable_array.o binary.o binary_io.o string_functions.o statistics.o file_functions.o hash_table.o polysample.o vector_class.o mol2_class.o cmdline_arguments.o keyword_class.o rotmatrix_class.o crystallography_class.o image_transforms.o quaternion_class.o zmatrix_class.o superimpose.o pnm_class.o
g95 -O2  -fstatic    -o raw2pgm  raw2pgm.f90   ps_routines.f fundamental_constants.o varying_string.o globals.o precision.o cartesian_class.o array_functions.o sort_functions.o variable_array.o binary.o binary_io.o string_functions.o statistics.o file_functions.o hash_table.o polysample.o vector_class.o mol2_class.o cmdline_arguments.o keyword_class.o rotmatrix_class.o crystallography_class.o image_transforms.o quaternion_class.o zmatrix_class.o superimpose.o pnm_class.o
mv raw2pgm $bindir
mv ni2pgm $bindir
mv  pgm2ni $bindir
mv  pgm2mask  $bindir
mv  chkmol2   $bindir
mv  mol2xyz   $bindir
mv  make_random_occ $bindir
mv  pgmcombine $bindir
mv  pgmave    $bindir
mv  zmat2mol2 $bindir
mv  zmat_anim $bindir
mv  zmatchk   $bindir
mv   zmat2xyz $bindir
mv   zmat_maker $bindir
mv   bin2gray   $bindir
mv   DZMC       $bindir
mv   ZMC       $bindir
mv   catmol2       $bindir
